@import"https://fonts.googleapis.com/css2?family=Inter:wght@300;400;500;600;700&display=swap";body{margin:0;font-family:-apple-system,BlinkMacSystemFont,Segoe UI,Roboto,Oxygen,Ubuntu,Cantarell,Fira Sans,Droid Sans,Helvetica Neue,sans-serif;-webkit-font-smoothing:antialiased;-moz-osx-font-smoothing:grayscale}code{font-family:source-code-pro,Menlo,Monaco,Consolas,Courier New,monospace}:root{--transition-speed: .2s;--transition-ease: ease-in-out;--ag-font-family: "Inter", sans-serif;--ag-row-hover-color: #f1f5f9;--ag-selected-row-background-color: #eff6ff;--ag-range-selection-border-color: #3b82f6;--ag-checkbox-checked-color: #3b82f6;--ag-borders: solid 1px #e2e8f0;--ag-border-color: #e2e8f0}body.dark{background-color:#020617;color:#f8fafc}body.dark{--ag-header-background-color: #0f172a;--ag-header-foreground-color: #94a3b8;--ag-row-hover-color: #1e293b;--ag-selected-row-background-color: rgba(59, 130, 246, .15);--ag-range-selection-border-color: #60a5fa;--ag-checkbox-checked-color: #60a5fa;--ag-borders: solid 1px #1e293b;--ag-border-color: #1e293b;--ag-background-color: #0f172a;--ag-foreground-color: #f8fafc;--ag-odd-row-background-color: #0f172a;--ag-control-panel-background-color: #0f172a;--ag-subheader-background-color: #1e293b}body{font-family:Inter,sans-serif;-webkit-font-smoothing:antialiased;-moz-osx-font-smoothing:grayscale;background-color:#f8fafc;margin:0;padding:0}html{scroll-behavior:smooth}::-webkit-scrollbar{width:8px;height:8px}::-webkit-scrollbar-track{background:transparent}::-webkit-scrollbar-thumb{background:#cbd5e1;border-radius:4px}::-webkit-scrollbar-thumb:hover{background:#94a3b8}.fade-in{animation:fadeIn .3s ease-in-out}.slide-up{animation:slideUp .3s ease-in-out}@keyframes fadeIn{0%{opacity:0}to{opacity:1}}@keyframes slideUp{0%{transform:translateY(10px);opacity:0}to{transform:translateY(0);opacity:1}}.page-enter{opacity:0}.ag-header-cell.ag-multiline-header .ag-header-cell-label{align-items:flex-start}.ag-header-cell.ag-multiline-header .ag-header-cell-text{white-space:normal;overflow-wrap:anywhere;line-height:1.2;display:-webkit-box;-webkit-box-orient:vertical;-webkit-line-clamp:3;overflow:hidden}.calculation-loading-spinner:hover{cursor:pointer}.file-field-wrapper{display:flex;flex-direction:column;align-items:flex-start;justify-content:center;gap:8px;width:100%;min-width:0;text-align:left}.file-field-name{margin:0;width:100%;font-size:.95rem;line-height:1.2;overflow:hidden;text-overflow:ellipsis;white-space:nowrap}.file-field-actions{display:flex;align-items:center;gap:8px;flex-wrap:wrap}.ag-menu-option.custom-export-item{display:flex;align-items:center;padding:4px 12px;cursor:pointer}.ag-menu-option.custom-export-item:hover{background-color:var(--ag-menu-option-hover-background-color, #e6e6e6)}.ag-icon-export{display:inline-block;width:16px;height:16px;margin-right:8px;background-image:url(data:image/svg+xml;base64,PHN2ZyB3aWR0aD0nMjQnIGhlaWdodD0nMjQnIHZpZXdCb3g9JzAgMCAyNCAyNCcgeG1sbnM9J2h0dHA6Ly93d3cudzMub3JnLzIwMDAvc3ZnJz48cGF0aCBmaWxsPSdub25lJyBzdHJva2U9JyM0NDQ0NDQnIHN0cm9rZS13aWR0aD0nMicgc3Ryb2tlLWxpbmVjYXA9J3JvdW5kJyBzdHJva2UtbGluZWpvaW49J3JvdW5kJyBkPSJNMTIgM3Y5bTAtOWwtNCA0bTQgLTRsNCA0bS00IDE0YTIgMiAwIDAgMCAyIDJoMTRhMiAyIDAgMCAwIDItMnYtNCIvPjwvc3ZnPg==);background-repeat:no-repeat;background-position:center;background-size:contain}
