# Generated by Django 6.0.3 on 2026-03-26 16:10

import lex.core.models.LexModel
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("audit_logging", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="auditlog",
            name="created_at",
            field=models.DateTimeField(
                default=lex.core.models.LexModel.lex_datetime_now, editable=False
            ),
        ),
        migrations.AddField(
            model_name="auditlog",
            name="edited_at",
            field=models.DateTimeField(
                default=lex.core.models.LexModel.lex_datetime_now, editable=False
            ),
        ),
        migrations.AddField(
            model_name="auditlogstatus",
            name="edited_at",
            field=models.DateTimeField(
                default=lex.core.models.LexModel.lex_datetime_now, editable=False
            ),
        ),
    ]
